@@ -11,7 +11,7 @@ if ($SCRIPT_DIR eq '' && open(F, '/etc/openwebmail_path.conf')) {
 if ($SCRIPT_DIR eq '') { print "Content-type: text/html\n\nSCRIPT_DIR not set in /etc/openwebmail_path.conf !\n"; exit 0; }
 push (@INC, $SCRIPT_DIR);
 
-foreach (qw(PATH ENV BASH_ENV CDPATH IFS TERM)) { $ENV{$_}='' }	# secure ENV
+foreach (qw(ENV BASH_ENV CDPATH IFS TERM)) {delete $ENV{$_}}; $ENV{PATH}='/bin:/usr/bin'; # secure ENV
 umask(0002); # make sure the openwebmail group can write
 
 use strict;
@@ -25,6 +25,7 @@ require "modules/filelock.pl";
 require "modules/tool.pl";
 require "modules/datetime.pl";
 require "modules/lang.pl";
+require "modules/mime.pl";
 require "auth/auth.pl";
 require "quota/quota.pl";
 require "shares/ow-shared.pl";
@@ -155,9 +156,9 @@ sub addressbook {
       openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
 
       foreach (@namelist) {
-         delete $addresses{$_} if ( ! is_entry_matched($abook_keyword,$abook_searchtype, $_, $notes{$_}, $addresses{$_}) );
+         delete $addresses{$_} if (!is_entry_matched($abook_keyword,$abook_searchtype,
+                                                     $_, $notes{$_}, $addresses{$_}));
       }
-
       foreach my $name (sort { lc($a) cmp lc($b) } keys %addresses) {
          my $email=$addresses{$name};
          my $emailstr;
@@ -221,6 +222,8 @@ sub addressbook {
       openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
 
       foreach my $name (@namelist) {
+         next if (!is_entry_matched($abook_keyword,$abook_searchtype,
+                                    $name, $globalnotes{$name}, $globaladdresses{$name}));
          my $email=$globaladdresses{$name};
          my $emailstr;
 
@@ -272,7 +275,7 @@ sub addressbook {
    # rebuild entries not checked on address popup window backto elist
    my @u=sort values(%emailhash);
 
-   $elist=join(",", @u); 
+   $elist=join(",", @u);
    $elist=ow::tool::escapeURL($elist);	# escape elist to make it a safe string in HTML
 
    $temphtml = ow::tool::hiddens(elist=>$elist,
@@ -343,7 +346,7 @@ sub importabook {
          open (ABOOK, ">>$addrbookfile"); # Create if nonexistent
          close(ABOOK);
       }
-      ow::filelock::lock($addrbookfile, LOCK_EX|LOCK_NB) or
+      ow::filelock::lock($addrbookfile, LOCK_EX) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
 
       my ($stat,$err,@namelist)=read_abook($addrbookfile,\%addresses,\%notes);
@@ -449,7 +452,7 @@ sub importabook {
 sub exportabook {
    my $addrbookfile=dotpath('address.book');
 
-   ow::filelock::lock($addrbookfile, LOCK_EX|LOCK_NB) or
+   ow::filelock::lock($addrbookfile, LOCK_EX) or
       openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
 
    my %addresses=();
@@ -470,7 +473,9 @@ sub exportabook {
    print qq|\n|;
    print qq|Name,E-mail Address,Note\n|;
    foreach (@namelist) {
-      print "$_,$addresses{$_},$notes{$_}\n";
+      my @line = ($_, $addresses{$_}, $notes{$_}); 
+      map { $_ = qq|"$_"| if /,/; } @line; # quote str if contains comma
+      print join(",", @line)."\n"; 
    }
 
    writelog("export addressbook");
@@ -491,25 +496,34 @@ sub importabook_pine {
    }
 
    if (open (PINEBOOK,"$homedir/.addressbook") ) {
+      my @lines=();
       my ($name, $email, $note);
       my (%addresses, %notes);
       my $abooktowrite='';
 
-      ow::filelock::lock($addrbookfile, LOCK_EX|LOCK_NB) or
+      ow::filelock::lock($addrbookfile, LOCK_EX) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
 
       my ($stat,$err,@namelist)=read_abook($addrbookfile, \%addresses, \%notes);
       openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
 
       while (<PINEBOOK>) {
-         my ($name, $email, $note) = (split(/\t/, $_,5))[1,2,4];
-         chomp($email);
-         chomp($note);
-         next if ($email=~/^\s*$/);  # skip if email is null
-         $addresses{"$name"} = $email;
-         $notes{"$name"}=$note;
+         chomp;
+         if (/^ / && $#lines>=0) {
+            s/^ //;
+            $lines[$#lines].=$_;
+         } else {
+            push(@lines, $_);
+         }
       }
       close (PINEBOOK);
+      foreach (@lines) {
+         my ($name, $email, $note) = (split(/\t/, $_,5))[1,2,4];
+         next if ($email=~/^\s*$/);  # skip if email is null
+         $name=decode_mimewords($name);
+         $addresses{"$name"} = decode_mimewords($email);
+         $notes{"$name"}=decode_mimewords($note);
+      }
 
       # replace the address book
       ($stat,$err)=write_abook($addrbookfile,$config{'maxbooksize'},\%addresses,\%notes);
@@ -527,7 +541,7 @@ sub exportabook_pine {
    my $addrbookfile=dotpath('address.book');
 
    if (-f $addrbookfile) {
-      ow::filelock::lock($addrbookfile, LOCK_SH) or
+      ow::filelock::lock($addrbookfile, LOCK_SH|LOCK_NB) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
 
       my (%nicknames, %emails, %fccs, %notes);
@@ -541,7 +555,7 @@ sub exportabook_pine {
 
       ow::filelock::lock("$homedir/.addressbook", LOCK_EX) or
          openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $homedir/.addressbook!");
-           
+
       if (open (PINEBOOK, "$homedir/.addressbook")) {
          while (<PINEBOOK>) {
             my ($nickname, $name, $email, $fcc, $note) = (split(/\t/, $_,5))[1,2,4];
@@ -794,40 +808,42 @@ sub modaddress {
 
    if (($realname && $address) || (($mode eq 'delete') && $realname) ) {
 
-      my (%addresses, %notes, $name, $email, $note);
       my $addrbookfile=dotpath('address.book');
+      my ($stat,$err,@namelist);
+      my (%addresses, %notes, $name, $email, $note);
 
-      if ( -f $addrbookfile ) {
-         if ($mode ne 'delete') {
-            if ( (-s $addrbookfile) >= ($config{'maxbooksize'} * 1024) ) {
-               openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_toobig'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>$lang_err{'tryagain'}|);
-            }
+      ow::filelock::lock($addrbookfile, LOCK_EX) or
+         openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
+
+      if (! -z $addrbookfile ) {
+         ($stat,$err,@namelist)=read_abook($addrbookfile, \%addresses, \%notes);
+         if ($stat<0) {
+            ow::filelock::lock($addrbookfile, LOCK_UN);
+            openwebmailerror(__FILE__, __LINE__, $err);
          }
-         ow::filelock::lock($addrbookfile, LOCK_EX|LOCK_NB) or
-            openwebmailerror(__FILE__, __LINE__, "$lang_err{'couldnt_lock'} $addrbookfile!");
-         my ($stat,$err,@namelist)=read_abook($addrbookfile, \%addresses, \%notes);
-         openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
-
-         if ($mode eq 'delete') {
-            delete $addresses{"$realname"};
-         } else {
-            $addresses{"$realname"} = $address;
-            # overwrite old note only if new one is not _reserved_
-            # check addaddress in openwebmail-read.pl
-            if ($usernote ne '_reserved_') {
-               $notes{"$realname"} = $usernote;
-            }
-         }
-
-         # replace the address book
-         my ($stat,$err)=write_abook($addrbookfile,$config{'maxbooksize'},\%addresses,\%notes);
-         openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
-         ow::filelock::lock($addrbookfile, LOCK_UN);
-      } else {
-         # create a new address book
-         my ($stat,$err)=write_abook($addrbookfile,$config{'maxbooksize'},\%addresses,\%notes);
-         openwebmailerror(__FILE__, __LINE__, $err) if ($stat<0);
       }
+
+      if ($mode eq 'delete') {
+         delete $addresses{"$realname"};
+      } else {
+         if ( (-s $addrbookfile) >= ($config{'maxbooksize'} * 1024) ) {
+            ow::filelock::lock($addrbookfile, LOCK_UN);
+            openwebmailerror(__FILE__, __LINE__, qq|$lang_err{'abook_toobig'} <a href="$config{'ow_cgiurl'}/openwebmail-abook.pl?action=editaddresses&amp;sessionid=$thissession&amp;sort=$sort&amp;folder=$escapedfolder&amp;page=$page&amp;message_id=$escapedmessageid">$lang_err{'back'}</a>$lang_err{'tryagain'}|);
+         }
+         $addresses{"$realname"} = $address;
+         # overwrite old note only if new one is not _reserved_
+         # check addaddress in openwebmail-read.pl
+         $notes{"$realname"} = $usernote if ($usernote ne '_reserved_');
+      }
+
+      # replace the address book
+      ($stat,$err)=write_abook($addrbookfile,$config{'maxbooksize'},\%addresses,\%notes);
+      if ($stat<0) {
+         ow::filelock::lock($addrbookfile, LOCK_UN) or
+         openwebmailerror(__FILE__, __LINE__, $err);
+      }
+
+      ow::filelock::lock($addrbookfile, LOCK_UN);
    }
 
    if ( param('message_id') ) {
